==> xdatabase-1.0.5.tar.gz: lib/src/xdatabase_base.dart <==
// TODO: Put public facing types in this file.

import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'dart:mirrors';

import 'package:path/path.dart';

import 'table_meta.dart';

String symbolToName(Symbol symbol) {
  final bs = symbol.toString();
  return bs.substring(8, bs.length - 2);
}

String nameUnderLine(String original) {
  return original
      .split('')
      .map((w) {
        if (w.toLowerCase() == w) {
          return w;
        } else {
          return '_${w.toLowerCase()}';
        }
      })
      .toList()
      .join('');
}

class Table {
  final String tableName;
  const Table(this.tableName);
}

class Index {
  final String name;
  final List<String> columns;
  const Index({required this.name, required this.columns});
}

class UniqueIndex {
  final String name;
  final List<String> columns;
  const UniqueIndex({required this.name, required this.columns});
}

class Field {
  final String type;
  final bool primary;
  final String mark;
  const Field({required this.type, this.primary = false, this.mark = ''});
}

class DeclarationHolder {
  Declarations declarations;
  Map<Type, TableDeclaration> tables;
  DeclarationHolder._privateConstructor()
      : declarations = Declarations(),
        tables = <Type, TableDeclaration>{};
  static final DeclarationHolder _instance =
      DeclarationHolder._privateConstructor();
  factory DeclarationHolder() {
    return _instance;
  }

  TableDeclaration? ofTable<T>() {
    if (tables.containsKey(T)) {
      return tables[T]!;
    }
    return null;
  }

  void registTable(Type tp) {
    final cm = reflectClass(tp);

    var tableName = '';
    var indexes = <IndexDeclaration>[];
    var uniqueIndexes = <UniqueDeclaration>[];
    var fields = <FieldDeclaration>[];

    cm.metadata.forEach((InstanceMirror element) {
      if (element.reflectee is Table) {
        final table = element.reflectee as Table;
        tableName = table.tableName;
      }
      if (element.reflectee is Index) {
        final index = element.reflectee as Index;
        indexes.add(
            IndexDeclaration(indexName: index.name, fields: index.columns));
      }
      if (element.reflectee is UniqueIndex) {
        final unique = element.reflectee as UniqueIndex;
        uniqueIndexes.add(
            UniqueDeclaration(indexName: unique.name, fields: unique.columns));
      }
    });

    cm.declarations.forEach((Symbol key, DeclarationMirror value) {
      value.metadata.forEach((InstanceMirror element) {
        if (element.reflectee is Field) {
          final field = element.reflectee as Field;
          fields.add(FieldDeclaration(
              symbol: key,
              fieldName: nameUnderLine(symbolToName(key)),
              fieldType: field.type,
              primary: field.primary,
              mark: field.mark));
        }
      });
    });
    var table = TableDeclaration(tableName: tableName);
    table.fields.addAll(fields);
    table.indexes.addAll(indexes);
    table.uniqueIndexes.addAll(uniqueIndexes);
    declarations.tables.addEntries([MapEntry(tableName, table)]);
    tables.addEntries([MapEntry(tp, table)]);
  }

  void saveToPath(String path) {
    final map = declarations.asMap();
    File(path).writeAsStringSync(jsonEncode(map));
  }

  String compareTo(Declarations other) {
    final tableNeedsToCreate = declarations.tables.keys
        .where((e) => !other.tables.containsKey(e))
        .toList();
    final tableNeedsToDelete = other.tables.keys
        .where((element) => !declarations.tables.containsKey(element))
        .toList();
    var lines = [];
    lines.addAll(tableNeedsToCreate
        .map((tableName) => declarations.tables[tableName]!.createScript())
        .toList());
    lines.addAll(tableNeedsToDelete
        .map((tableName) => other.tables[tableName]!.dropTableSql()));
    for (var tableDeclaration in declarations.tables.values) {
      for (var currentDeclaration in other.tables.values) {
        if (tableDeclaration.tableName == currentDeclaration.tableName) {
          lines.add(tableDeclaration.compareWith(currentDeclaration));
        }
      }
    }
    return lines.join('\n\n');
  }

  String compareWithPath(String path) {
    final declarationFile = File(path);
    if (!declarationFile.existsSync()) {
      return declarations.createTables();
    }
    final jtext = declarationFile.readAsStringSync();
    final defineMap = jsonDecode(jtext);
    final currentDeclaration = Declarations.fromMap(defineMap);
    return compareTo(currentDeclaration);
  }
}

class Migration {
  List<Type> tables;
  Migration({required this.tables}) {
    for (var tb in tables) {
      DeclarationHolder().registTable(tb);
    }
  }

  String get workingDirectory {
    return dirname(Platform.script.toString()).substring(7);
  }

  String get versionsDirectory {
    final basePath = workingDirectory;
    final dirPath = join(basePath, 'versions');
    final dir = Directory(dirPath);
    if (!dir.existsSync()) {
      dir.createSync();
    }
    return dirPath;
  }

  String versionInFilePath(String filePath) {
    final fileName = filePath.split(separator).last;
    return fileName.split('.').first;
  }

  String get latestVersionPath {
    final dir = Directory(versionsDirectory);
    final fileList = dir.listSync();
    final versions = fileList
        .where((f) => f.path.endsWith('.map'))
        .map((f) => versionInFilePath(f.path))
        .toList();
    versions.sort((v1, v2) => int.parse(v1).compareTo(int.parse(v2)));
    final matched = fileList
        .where((element) => element.path.endsWith('.map'))
        .where((element) => versionInFilePath(element.path) == versions.last)
        .map((e) => e.path)
        .toList();
    return matched.isNotEmpty ? matched.first : '';
  }

  String get currentUpgradeSqlPath {
    return join(versionsDirectory, 'current.sql');
  }

  String get currentVersionPath {
    var fileName = newVersion();
    return join(versionsDirectory, '$fileName.map');
  }

  String targetVersionUpgradeSQLPath(String versionPath) {
    final ver = versionInFilePath(versionPath);
    return join(versionsDirectory, '$ver.sql');
  }

  int newVersion() {
    final now = DateTime.now();
    final fileName = now.year * 10000000000 +
        now.month * 100000000 +
        now.day * 1000000 +
        now.hour * 10000 +
        now.minute * 100 +
        now.second;
    return fileName;
  }

  void diffVersion() {
    print('newest path:$latestVersionPath');
    final sql = DeclarationHolder().compareWithPath(latestVersionPath);
    final f = File(currentUpgradeSqlPath);
    f.writeAsStringSync(sql);
  }

  void releaseVersion() {
    final vpath = currentVersionPath;
    DeclarationHolder().saveToPath(vpath);
    File(currentUpgradeSqlPath).copySync(targetVersionUpgradeSQLPath(vpath));
    File(currentUpgradeSqlPath).deleteSync();
  }

  void executeCommand(String command) {
    switch (command.toLowerCase()) {
      case 'diff':
        diffVersion();
        break;
      case 'release':
        releaseVersion();
        break;
      default:
        print('command in (diff, release)');
        break;
    }
  }
}

==> xdatabase-1.0.8.tar.gz: lib/src/xdatabase_base.dart <==
// TODO: Put public facing types in this file.

import 'dart:convert';
import 'dart:core';
import 'dart:io';

import 'package:xdatabase/src/connection_pool.dart';

import 'table_meta.dart';

abstract class Bean<M> {
  Map<String, dynamic> asMap();
  M fromMap(Map<String, dynamic> map);
  Future<int> insert(QueryAble conn);
  Future<int> update(String statement, QueryAble conn);
  Future<int> delete(String statement, QueryAble conn);
  Future<int> updateByPK(QueryAble conn);
  Future<int> deleteByPk(QueryAble conn);
  Future<M?> selectByPk(QueryAble conn);
  Future<List<M>> select(String statement, QueryAble conn);
}

String symbolToName(Symbol symbol) {
  final bs = symbol.toString();
  return bs.substring(8, bs.length - 2);
}

String nameUnderLine(String original) {
  return original
      .split('')
      .map((w) {
        if (w.toLowerCase() == w) {
          return w;
        } else {
          return '_${w.toLowerCase()}';
        }
      })
      .toList()
      .join('');
}

class Table {
  final String tableName;
  const Table(this.tableName);
}

class Index {
  final String name;
  final List<String> columns;
  const Index({required this.name, required this.columns});
}

class UniqueIndex {
  final String name;
  final List<String> columns;
  const UniqueIndex({required this.name, required this.columns});
}

class Field {
  final String type;
  final bool primary;
  final String mark;
  const Field({required this.type, this.primary = false, this.mark = ''});
}

class DeclarationHolder {
  Declarations declarations;
  Map<Type, TableDeclaration> tables;
  Map<String, TableDeclaration> queryTables;
  DeclarationHolder._privateConstructor()
      : declarations = Declarations(),
        tables = <Type, TableDeclaration>{},
        queryTables = <String, TableDeclaration>{};
  static final DeclarationHolder _instance =
      DeclarationHolder._privateConstructor();
  factory DeclarationHolder() {
    return _instance;
  }

  TableDeclaration? ofTable<T>() {
    if (tables.containsKey(T)) {
      return tables[T]!;
    }
    return null;
  }

  TableDeclaration? byTable(String className) {
    if (queryTables.containsKey(className)) {
      return queryTables[className];
    }
  }

  void saveToPath(String path) {
    final map = declarations.asMap();
    File(path).writeAsStringSync(jsonEncode(map));
  }

  void fromPersists(String persists) {
    final map = jsonDecode(persists);
    declarations = Declarations.fromMap(map);
    declarations.tables.forEach((tableName, tableDeclaration) {
      queryTables[tableDeclaration.className] = tableDeclaration;
    });
  }

  String compareTo(Declarations other) {
    final tableNeedsToCreate = declarations.tables.keys
        .where((e) => !other.tables.containsKey(e))
        .toList();
    final tableNeedsToDelete = other.tables.keys
        .where((element) => !declarations.tables.containsKey(element))
        .toList();
    var lines = [];
    lines.addAll(tableNeedsToCreate
        .map((tableName) => declarations.tables[tableName]!.createScript())
        .toList());
    lines.addAll(tableNeedsToDelete
        .map((tableName) => other.tables[tableName]!.dropTableSql()));
    for (var tableDeclaration in declarations.tables.values) {
      for (var currentDeclaration in other.tables.values) {
        if (tableDeclaration.tableName == currentDeclaration.tableName) {
          lines.add(tableDeclaration.compareWith(currentDeclaration));
        }
      }
    }
    return lines.join('\n\n');
  }

  String compareWithPath(String path) {
    final declarationFile = File(path);
    if (!declarationFile.existsSync()) {
      return declarations.createTables();
    }
    final jtext = declarationFile.readAsStringSync();
    final defineMap = jsonDecode(jtext);
    final currentDeclaration = Declarations.fromMap(defineMap);
    return compareTo(currentDeclaration);
  }
}
